@@ -398,8 +398,12 @@ class AWS4Auth(AuthBase):
 
         # encode body and generate body hash
         if hasattr(req, 'body') and req.body is not None:
+            if hasattr(req.body, 'read'):
+                req.body = req.body.read()
             self.encode_body(req)
             content_hash = hashlib.sha256(req.body)
+        elif hasattr(req, 'content') and req.content is not None:
+            content_hash = hashlib.sha256(req.content)
         else:
             content_hash = hashlib.sha256(b'')
         req.headers['x-amz-content-sha256'] = content_hash.hexdigest()
@@ -562,7 +566,7 @@ class AWS4Auth(AuthBase):
         """
         Create the AWS authentication Canonical Request string.
 
-        req            -- Requests PreparedRequest object. Should already
+        req            -- Requests/Httpx PreparedRequest object. Should already
                           include an x-amz-content-sha256 header
         cano_headers   -- Canonical Headers section of Canonical Request, as
                           returned by get_canonical_headers()
@@ -570,11 +574,12 @@ class AWS4Auth(AuthBase):
                           get_canonical_headers()
 
         """
-        url = urlparse(req.url)
+        raw_url = str(req.url) # in case the url property is of type URL
+        url = urlparse(raw_url)
         path = self.amz_cano_path(url.path)
         # AWS handles "extreme" querystrings differently to urlparse
         # (see post-vanilla-query-nonunreserved test in aws_testsuite)
-        split = req.url.split('?', 1)
+        split = raw_url.split('?', 1)
         qs = split[1] if len(split) == 2 else ''
         qs = self.amz_cano_querystring(qs)
         payload_hash = req.headers['x-amz-content-sha256']
@@ -610,7 +615,7 @@ class AWS4Auth(AuthBase):
         # in the signed headers, but Requests doesn't include it in a
         # PreparedRequest
         if 'host' not in headers:
-            headers['host'] = urlparse(req.url).netloc.split(':')[0]
+            headers['host'] = urlparse(str(req.url)).netloc.split(':')[0]
         # Aggregate for upper/lowercase header name collisions in header names,
         # AMZ requires values of colliding headers be concatenated into a
         # single header with lowercase name.  Although this is not possible with
@@ -701,18 +706,16 @@ class AWS4Auth(AuthBase):
         qs -- querystring
 
         """
-        safe_qs_amz_chars = '&=+'
         safe_qs_unresvd = '-_.~'
         # If Python 2, switch to working entirely in str
         # as quote() has problems with Unicode
         if PY2:
             qs = qs.encode('utf-8')
-            safe_qs_amz_chars = safe_qs_amz_chars.encode()
             safe_qs_unresvd = safe_qs_unresvd.encode()
-        qs = unquote(qs)
         space = b' ' if PY2 else ' '
         qs = qs.split(space)[0]
-        qs = quote(qs, safe=safe_qs_amz_chars)
+        # prevent parse_qs from interpreting semicolon as an alternative delimiter to ampersand
+        qs = qs.replace(';', '%3B')
         qs_items = {}
         for name, vals in parse_qs(qs, keep_blank_values=True).items():
             name = quote(name, safe=safe_qs_unresvd)
